//
//  NibResource+Generator.swift
//  
//
//  Created by Tom Lokhorst on 2022-06-24.
//

import Foundation
import RswiftResources

extension NibResource {
    public static func generateStruct(nibs: [NibResource], prefix: SwiftIdentifier) -> Struct {
        let structName = SwiftIdentifier(name: "nib")
        let qualifiedName = prefix + structName

        let warning: (String) -> Void = { print("warning: [R.swift]", $0) }

        // Unify different localizations of nibs
        let unifiedNibs = unifyLocalizations(nibs: nibs, warning: warning)

        let groupedNibs = unifiedNibs.grouped(bySwiftIdentifier: \.name)
        groupedNibs.reportWarningsForDuplicatesAndEmpties(source: "xib", result: "file", warning: warning)

        let vargetters = groupedNibs.uniques
            .map { $0.generateVarGetter() }
            .sorted { $0.name < $1.name }

        let comments = ["This `\(qualifiedName.value)` struct is generated, and contains static references to \(vargetters.count) nibs."]

        // additional module references, for use in validate function
        let additionalModuleReferences = Set(nibs.map { $0.isAppKit ? ModuleReference.appKit : ModuleReference.uiKit })

        return Struct(comments: comments, name: structName, additionalModuleReferences: additionalModuleReferences) {
            Init.bundle

            vargetters

            if groupedNibs.uniques.count > 0 {
                generateValidate(nibs: groupedNibs.uniques)
            }
        }
    }

    private static func generateValidate(nibs: [NibResource]) -> Function {
        Function(
            comments: [],
            name: .init(name: "validate"),
            params: [],
            returnThrows: true,
            returnType: .void,
            valueCodeString: nibs.flatMap { $0.generateValidateLines() }.joined(separator: "\n")
        )
    }

    private static func unifyLocalizations(nibs: [NibResource], warning: (String) -> Void) -> [NibResource] {
        var result: [NibResource] = []

        for localizations in Dictionary(grouping: nibs, by: \.name).values {
            guard let nib = localizations.first else { continue }
            let ur = nib.unify(localizations: localizations)

            let diffs: [String] = [
                ur.differentNames ? "names" : nil,
                ur.differentRootViews ? "root views" : nil,
                ur.differentInitialReusables ? "initial reusables" : nil,
                ur.differentDeploymentTargets ? "deployment targets" : nil,
            ].compactMap { $0 }

            if diffs.count > 0 {
                warning("Skipping generation of nib '\(nib.name)', because \(diffs.joined(separator: ", ")) don't match in all localizations")
                continue
            }
            result.append(ur.resource)
        }

        return result
    }
}

extension NibResource {
    var genericTypeReference: TypeReference {
        TypeReference(
            module: .rswiftResources,
            name: "NibReference",
            genericArgs: [rootViews.first ?? TypeReference.uiView]
        )
    }

    func generateVarGetter() -> VarGetter {
        if let reusable = reusables.first {
            let typeReference = TypeReference(
                module: .rswiftResources,
                name: "NibReferenceReuseIdentifier",
                genericArgs: [rootViews.first ?? TypeReference.uiView, reusable.type]
            )
            return VarGetter(
                comments: ["Nib `\(name)`."],
                deploymentTarget: deploymentTarget,
                name: SwiftIdentifier(name: name),
                typeReference: typeReference,
                valueCodeString: ".init(name: \"\(name.escapedStringLiteral)\", bundle: bundle, identifier: \"\(reusable.identifier.escapedStringLiteral)\")"
            )
        } else {
            let typeReference = TypeReference(
                module: .rswiftResources,
                name: "NibReference",
                genericArgs: [rootViews.first ?? TypeReference.uiView]
            )
            return VarGetter(
                comments: ["Nib `\(name)`."],
                deploymentTarget: deploymentTarget,
                name: SwiftIdentifier(name: name),
                typeReference: typeReference,
                valueCodeString: ".init(name: \"\(name.escapedStringLiteral)\", bundle: bundle)"
            )
        }
    }

    func generateValidateLines() -> [String] {
        let validateImagesLines = self.usedImageIdentifiers.uniqueAndSorted()
            .map { nameCatalog -> String in
                if isAppKit {
                    if nameCatalog.isSystemCatalog {
                        return "if #available(macOS 11.0, *) { if AppKit.NSImage(systemSymbolName: \"\(nameCatalog.name)\", accessibilityDescription: nil) == nil { throw RswiftResources.ValidationError(\"[R.swift] System image named '\(nameCatalog.name)' is used in nib '\(self.name)', but couldn't be loaded.\") } }"
                    } else {
                        return "if bundle.image(forResource: \"\(nameCatalog.name)\") == nil { throw RswiftResources.ValidationError(\"[R.swift] Image named '\(nameCatalog.name)' is used in nib '\(self.name)', but couldn't be loaded.\") }"
                    }
                } else {
                    if nameCatalog.isSystemCatalog {
                        return "if #available(iOS 13.0, *) { if UIKit.UIImage(systemName: \"\(nameCatalog.name)\") == nil { throw RswiftResources.ValidationError(\"[R.swift] System image named '\(nameCatalog.name)' is used in nib '\(self.name)', but couldn't be loaded.\") } }"
                    } else {
                        return "if UIKit.UIImage(named: \"\(nameCatalog.name)\", in: bundle, compatibleWith: nil) == nil { throw RswiftResources.ValidationError(\"[R.swift] Image named '\(nameCatalog.name)' is used in nib '\(self.name)', but couldn't be loaded.\") }"
                    }
                }
            }

        let validateColorLines = self.usedColorResources.uniqueAndSorted()
            .filter { !$0.isSystemCatalog }
            .map { nameCatalog in
                isAppKit
                ? "if AppKit.NSColor(named: \"\(nameCatalog.name)\", bundle: bundle) == nil { throw RswiftResources.ValidationError(\"[R.swift] Color named '\(nameCatalog.name)' is used in nib '\(self.name)', but couldn't be loaded.\") }"
                : "if UIKit.UIColor(named: \"\(nameCatalog.name)\", in: bundle, compatibleWith: nil) == nil { throw RswiftResources.ValidationError(\"[R.swift] Color named '\(nameCatalog.name)' is used in nib '\(self.name)', but couldn't be loaded.\") }"
            }


        return (validateImagesLines + validateColorLines)
    }
}
